/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "SWAlgorithmPlugin.h"

#include "SWAlgorithmTask.h"
#include "SWTaskFactory.h"

#include <util_gui/GUIUtils.h>
#include <util_ov_annotated_dna/AnnotatedDNAView.h>
#include <util_ov_annotated_dna/ADVConstants.h>
#include <gobjects/AnnotationTableObject.h>

#include <test_framework/xmltest/XMLTestFormat.h>
#include <test_framework/GTest.h>
#include <test_framework/GTestFrameworkComponents.h>
#include <util_algorithm/GAutoDeleteList.h>

#include <core_api/AppContext.h>
#include <core_api/AppResources.h>
#include <core_api/CudaGpuRegistry.h>

#include <util_smith_waterman/SmithWatermanTests.h>
#include <util_ov_annotated_dna/ADVConstants.h>
#include <util_ov_annotated_dna/ADVSequenceObjectContext.h>
#include <util_ov_annotated_dna/ADVUtils.h>

#include <core_api/SmithWatermanTaskFactoryRegistry.h>

namespace GB2 {

extern "C" Q_DECL_EXPORT Plugin* GB2_PLUGIN_INIT_FUNC() {
#ifdef SW2_BUILD_WITH_SSE2
    bool runningOnSSEProc = AppResourcePool::isSSE2Enabled();
    if( !runningOnSSEProc ) {
        return 0;
    }
#endif
    SWAlgorithmPlugin * plug = new SWAlgorithmPlugin();
    return plug;
}

SWAlgorithmPlugin::SWAlgorithmPlugin() 
: Plugin(tr("Smith-Waterman SSE2"), tr("An implementation of Smith-Waterman algorithm with SSE2 instruction set support"))
{

    //Smith-Waterman algorithm tests
    GTestFormatRegistry * tfr = AppContext::getTestFramework()->getTestFormatRegistry();
    XMLTestFormat *xmlTestFormat = qobject_cast<XMLTestFormat*>(tfr->findFormat("XML"));
    assert(xmlTestFormat!=NULL);    

	GB2::GAutoDeleteList<GB2::XMLTestFactory>* l = new GB2::GAutoDeleteList<GB2::XMLTestFactory>(this);
	l->qlist = SWAlgorithmTests::createTestFactories();

    foreach(XMLTestFactory* f, l->qlist) { 
        bool res = xmlTestFormat->registerTestFactory(f);
        Q_UNUSED(res);
        assert(res);
    }	

	SmithWatermanTaskFactoryRegistry* swar = AppContext::getSmithWatermanTaskFactoryRegistry();
	swar->registerAlgorithm(new SWTaskFactory(classic), QString("Classic 2"));	
#ifdef SW2_BUILD_WITH_SSE2
	swar->registerAlgorithm(new SWTaskFactory(sse2), QString("SSE2"));
#endif
#ifdef SW2_BUILD_WITH_CUDA
	if ( AppContext::getCudaGpuRegistry()->empty() ) {
		swar->registerAlgorithm(new SWTaskFactory(cuda), QString("CUDA"));
	}
#endif
}

QList<XMLTestFactory*> SWAlgorithmTests::createTestFactories() {
    QList<XMLTestFactory*> res;
    res.append(GTest_SmithWatermnan::createFactory());
    return res;
}

} //namespace

