/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#include "Primer3Task.h"

namespace GB2 {

Primer3Task::Primer3Task(primer_args *pa, seq_args *sa): Task(tr("Pick primers task"), TaskFlag_None), pa(pa), sa(sa) {}
void Primer3Task::run(){           
    primers = runPrimer3(pa, sa);
}

Task::ReportResult Primer3Task::report() {
    return ReportResult_Finished;
}

pair_array_t Primer3Task::getBestPairs() {
    return primers.best_pairs;
}

primer_rec *Primer3Task::getLeftPrimers() {
    return primers.left;
}

primer_rec *Primer3Task::getRightPrimers() {
    return primers.right;
}

primer_rec *Primer3Task::getOligos() {
    return primers.intl;
}

Primer3Task::~Primer3Task(){
    if (NULL != sa->internal_input) free(sa->internal_input);
    if (NULL != sa->left_input) free(sa->left_input);
    if (NULL != sa->right_input) free(sa->right_input);    
    if (NULL != sa->sequence) free(sa->sequence);
    if (NULL != sa->quality)  free(sa->quality);
    if (NULL != sa->trimmed_seq) free(sa->trimmed_seq);

    /* edited by T. Koressaar for lowercase masking */
    if (NULL != sa->trimmed_orig_seq) free(sa->trimmed_orig_seq);

    if (NULL != sa->upcased_seq) free(sa->upcased_seq);
    if (NULL != sa->upcased_seq_r) free(sa->upcased_seq_r);
    if (NULL != sa->sequence_name) free(sa->sequence_name);
    if (NULL != sa->sequence_file) free(sa->sequence_file);    
    if (NULL != sa->error.data) free(sa->error.data);
    if (NULL != sa->warning.data) free(sa->warning.data); 

    if (primers.left != NULL) free(primers.left);
    if (primers.right != NULL) free(primers.right);
    if (primers.intl != NULL) free(primers.intl);
    if (primers.best_pairs.num_pairs > 0)        
        free(primers.best_pairs.pairs); 
    free(sa);
    free(pa);
}

}